"""
Armory SDK integrations for popular ML frameworks.

Available integrations:
- HuggingFace Transformers: ArmoryCallback for Trainer
- PyTorch Lightning: ArmoryLogger for Lightning
- Keras: ArmoryCallback for Keras fit()

Usage:
    from armory.integrations import ArmoryTrainerCallback

    trainer = Trainer(
        model=model,
        callbacks=[ArmoryTrainerCallback()],
    )
"""

from armory.integrations.huggingface import ArmoryTrainerCallback
from armory.integrations.pytorch_lightning import ArmoryLightningLogger
from armory.integrations.keras import ArmoryKerasCallback

__all__ = [
    "ArmoryTrainerCallback",
    "ArmoryLightningLogger",
    "ArmoryKerasCallback",
]
